//
//  LSMPNBuilder.h
//  Lightstreamer client for iOS UCA
//

#import <Foundation/Foundation.h>


/**
 Utility class that provides methods to build or parse the JSON structure used to represent the format of a push notification.
 
 It provides properties and methods to get and set the fields of a push notification, following the format specified by Apple's Push Notification service (APNs).
 This format is compatible with <[LSMPNSubscription notificationFormat]>.
 
 @see [LSMPNSubscription notificationFormat]
 */
@interface LSMPNBuilder : NSObject


/**
 @brief Creates an empty object to be used to create a push notification format from scratch.
 
 Use setters methods to set the value of push notification fields.
 */
- (nonnull instancetype) init;

/**
 @brief Creates an object based on the specified push notification format.
 
 Use properties and setter methods to get and set the value of push notification fields.
 
 @param notificationFormat A JSON structure representing a push notification format.

 @exception NSException if the notification is not a valid JSON structure.
 */
- (nonnull instancetype) initWithNotificationFormat:(nonnull NSString *)notificationFormat;

/**
 @brief Produces the JSON structure for the push notification format specified by this object.
 */
- (nonnull NSString *) build;

/**
 @brief Sets the `aps.alert` field.
 
 @param alert A string to be used for the `aps.alert` field value, or nil to clear it.
 */
- (nonnull LSMPNBuilder *) alert:(nullable NSString *)alert;

/**
 @brief Value of the `aps.alert` field.
 */
@property (nonatomic, readonly, nullable) NSString *alert;

/**
 @brief Sets the `aps.alert` field with an int value.
 
 @param badge An int to be used for the `aps.alert` field value.
 */
- (nonnull LSMPNBuilder *) badgeWithInt:(int)badge;

/**
 @brief Value of the `aps.badge` field as an int value.
 */
@property (nonatomic, readonly) int badgeAsInt;

/**
 @brief Sets the `aps.badge` field with a string value.
 
 @param badge A string to be used for the `aps.badge` field value, or nil to clear it.
 */
- (nonnull LSMPNBuilder *) badgeWithString:(nullable NSString *)badge;

/**
 @brief Value of the `aps.badge` field as a string value.
 */
@property (nonatomic, readonly, nullable) NSString *badgeAsString;

/**
 @brief Sets the `aps.alert.body` field.
 
 @param body A string to be used for the `aps.alert.body` field value, or nil to clear it.
 */
- (nonnull LSMPNBuilder *) body:(nullable NSString *)body;

/**
 @brief Value of the `aps.alert.body` field.
 */
@property (nonatomic, readonly, nullable) NSString *body;

/**
 @brief Sets the `aps.alert.loc-args` field.
 
 @param bodyLocArguments An array of strings to be used for the `aps.alert.loc-args` field value, or nil to clear it.
 */
- (nonnull LSMPNBuilder *) bodyLocArguments:(nullable NSArray<NSString *> *)bodyLocArguments;

/**
 @brief Value of the `aps.alert.loc-args` field as an array of strings.
 */
@property (nonatomic, readonly, nullable) NSArray<NSString *> *bodyLocArguments;

/**
 @brief Sets the `aps.alert.loc-key` field.
 
 @param bodyLocKey A string to be used for the `aps.alert.loc-key` field value, or nil to clear it.
 */
- (nonnull LSMPNBuilder *) bodyLocKey:(nullable NSString *)bodyLocKey;

/**
 @brief Value of the `aps.alert.loc-key` field.
 */
@property (nonatomic, readonly, nullable) NSString *bodyLocKey;

/**
 @brief Sets the `aps.category` field.
 
 @param category A string to be used for the `aps.category` field value, or nil to clear it.
 */
- (nonnull LSMPNBuilder *) category:(nullable NSString *)category;

/**
 @brief Value of the `aps.category` field.
 */
@property (nonatomic, readonly, nullable) NSString *category;

/**
 @brief Sets the `aps.content-available` field with an int value.
 
 @param contentAvailable An int to be used for the `aps.content-available` field value.
 */
- (nonnull LSMPNBuilder *) contentAvailableWithInt:(int)contentAvailable;

/**
 @brief Value of the `aps.content-available` field as an int value.
 */
@property (nonatomic, readonly) int contentAvailableAsInt;

/**
 @brief Sets the `aps.content-available` field with a string value.
 
 @param contentAvailable A string to be used for the `aps.content-available` field value, or nil to clear it.
 */
- (nonnull LSMPNBuilder *) contentAvailableWithString:(nullable NSString *)contentAvailable;

/**
 @brief Value of the `aps.content-available` field as a string value.
 */
@property (nonatomic, readonly, nullable) NSString *contentAvailableAsString;

/**
 @brief Sets the `aps.mutable-content` field with an int value.
 
 @param mutableContent An int to be used for the `aps.mutable-content` field value.
 */
- (nonnull LSMPNBuilder *) mutableContentWithInt:(int)mutableContent;

/**
 @brief Value of the `aps.mutable-content` field as an int value.
 */
@property (nonatomic, readonly) int mutableContentAsInt;

/**
 @brief Sets the `aps.mutable-content` field with a string value.
 
 @param mutableContent A string to be used for the `aps.mutable-content` field value, or nil to clear it.
 */
- (nonnull LSMPNBuilder *) mutableContentWithString:(nullable NSString *)mutableContent;

/**
 @brief Value of the `aps.mutable-content` field as a string value.
 */
@property (nonatomic, readonly, nullable) NSString *mutableContentAsString;

/**
 @brief Sets fields in the root of the notification format (excluding `aps`).
 
 @param customData A dictionary to be used for fields in the root of the notification format (excluding `aps`), or nil to clear them.
 */
- (nonnull LSMPNBuilder *) customData:(nullable NSDictionary<NSString *, id> *)customData;

/**
 @brief Fields in the root of the notification format (excluding `aps`).
 */
@property (nonatomic, readonly, nullable) NSDictionary<NSString *, id> *customData;

/**
 @brief Sets the `aps.alert.launch-image` field.
 
 @param launchImage A string to be used for the `aps.alert.launch-image` field value, or nil to clear it.
 */
- (nonnull LSMPNBuilder *) launchImage:(nullable NSString *)launchImage;

/**
 @brief Value of the `aps.alert.launch-image` field.
 */
@property (nonatomic, readonly, nullable) NSString *launchImage;

/**
 @brief Sets the `aps.alert.action-loc-key` field.
 
 @param locActionKey A string to be used for the `aps.alert.action-loc-key` field value, or nil to clear it.
 */
- (nonnull LSMPNBuilder *) locActionKey:(nullable NSString *)locActionKey;

/**
 @brief Value of the `aps.alert.action-loc-key` field.
 */
@property (nonatomic, readonly, nullable) NSString *locActionKey;

/**
 @brief Sets the `aps.sound` field.
 
 @param sound A string to be used for the `aps.sound` field value, or nil to clear it.
 */
- (nonnull LSMPNBuilder *) sound:(nullable NSString *)sound;

/**
 @brief Value of the `aps.sound` field.
 */
@property (nonatomic, readonly, nullable) NSString *sound;

/**
 @brief Sets the `aps.thread-id` field.
 
 @param threadId A string to be used for the `aps.thread-id` field value, or nil to clear it.
 */
- (nonnull LSMPNBuilder *) threadId:(nullable NSString *)threadId;

/**
 @brief Value of the `aps.thread-id` field.
 */
@property (nonatomic, readonly, nullable) NSString *threadId;

/**
 @brief Sets the `aps.alert.title` field.
 
 @param title A string to be used for the `aps.alert.title` field value, or nil to clear it.
 */
- (nonnull LSMPNBuilder *) title:(nullable NSString *)title;

/**
 @brief Value of the `aps.alert.title` field.
 */
@property (nonatomic, readonly, nullable) NSString *title;

/**
 @brief Sets the `aps.alert.subtitle` field.
 
 @param subtitle A string to be used for the `aps.alert.subtitle` field value, or nil to clear it.
 */
- (nonnull LSMPNBuilder *) subtitle:(nullable NSString *)subtitle;

/**
 @brief Value of the `aps.alert.subtitle` field.
 */
@property (nonatomic, readonly, nullable) NSString *subtitle;

/**
 @brief Sets the `aps.alert.title-loc-args` field.
 
 @param titleLocArguments An array of strings to be used for the `aps.alert.title-loc-args` field value, or nil to clear it.
 */
- (nonnull LSMPNBuilder *) titleLocArguments:(nullable NSArray<NSString *> *)titleLocArguments;

/**
 @brief Value of the `aps.alert.title-loc-args` field.
 */
@property (nonatomic, readonly, nullable) NSArray<NSString *> *titleLocArguments;

/**
 @brief Sets the `aps.alert.title-loc-key` field.
 
 @param titleLocKey A string to be used for the `aps.alert.title-loc-key` field value, or nil to clear it.
 */
- (nonnull LSMPNBuilder *) titleLocKey:(nullable NSString *)titleLocKey;

/**
 @brief Value of the `aps.alert.title-loc-key` field.
 */
@property (nonatomic, readonly, nullable) NSString *titleLocKey;


@end
