//
//  LSSubscriptionDelegate.h
//  Lightstreamer client for iOS UCA
//

#import <Foundation/Foundation.h>


@class LSSubscription;
@class LSItemUpdate;


/**
 Protocol to be implemented to receive <LSSubscription> events comprehending notifications of subscription/unsubscription, updates, errors and others.
 
 Events for these delegates are dispatched by a different thread than the one that generates them. This means that, upon reception of an event,
 it is possible that the internal state of the client has changed. On the other hand, all the notifications for a single <LSLightstreamerClient>, including
 notifications to <LSClientDelegate>, LSSubscriptionDelegate, <LSClientMessageDelegate>, <LSMPNDeviceDelegate> and <LSMPNSubscriptionDelegate>
 will be dispatched by the same thread.
 */
@protocol LSSubscriptionDelegate <NSObject>


@optional

/**
 @brief Event handler that is called by Lightstreamer each time a request to clear the snapshot pertaining to an item in the <LSSubscription> has been
 received from the Server.
 
 More precisely, this kind of request can occur in two cases:
 
 - For an item delivered in COMMAND mode, to notify that the state of the item becomes empty; this is equivalent to receiving an update carrying a
   DELETE command once for each key that is currently active.

 - For an item delivered in DISTINCT mode, to notify that all the previous updates received for the item should be considered as obsolete; hence, if the
   delegate were showing a list of recent updates for the item, it should clear the list in order to keep a coherent view.
 
 Note that, if the involved <LSSubscription> has a two-level behavior enabled (see <[LSSubscription commandSecondLevelFields]> and <[LSSubscription commandSecondLevelFieldSchema]>),
 the notification refers to the first-level item (which is in COMMAND mode). This kind of notification is not possible for second-level items (which are in MERGE mode). 
 
 @param subscription The <LSSubscription> involved.
 
 @param itemName The name of the involved item. If the <LSSubscription> was initialized using an "Item Group" then a nil value is supplied.
 
 @param itemPos The 1-based position of the item within the "Item List" or "Item Group".
 */
- (void) subscription:(nonnull LSSubscription *)subscription didClearSnapshotForItemName:(nullable NSString *)itemName itemPos:(NSUInteger)itemPos;

/**
 @brief Event handler that is called by Lightstreamer to notify that, due to internal resource limitations, Lightstreamer Server dropped one or more updates 
 for an item that was subscribed to as a second-level subscription.
 
 Such notifications are sent only if the <LSSubscription> was configured in unfiltered mode (second-level items are always in `MERGE` mode and inherit
 the frequency configuration from the first-level Subscription).
 
 By implementing this method it is possible to perform recovery actions.
 
 @param subscription The <LSSubscription> involved.
 
 @param lostUpdates The number of consecutive updates dropped for the item.
 
 @param key The value of the key that identifies the second-level item.
 
 @see [LSSubscription requestedMaxFrequency]
 
 @see [LSSubscription commandSecondLevelFields]
 
 @see [LSSubscription commandSecondLevelFieldSchema]
 */
- (void) subscription:(nonnull LSSubscription *)subscription didLoseUpdates:(NSUInteger)lostUpdates forCommandSecondLevelItemWithKey:(nonnull NSString *)key;

/**
 @brief Event handler that is called when the Server notifies an error on a second-level subscription.
 
 The error code can be one of the following:
 
 - 14 - the key value is not a valid name for the Item to be subscribed; only in this case, the error is detected directly by the library before issuing
   the actual request to the Server
 
 - 17 - bad Data Adapter name or default Data Adapter not defined for the current Adapter Set
 
 - 21 - bad Group name
 
 - 22 - bad Group name for this Schema
 
 - 23 - bad Schema name
 
 - 24 - mode not allowed for an Item
 
 - 26 - unfiltered dispatching not allowed for an Item, because a frequency limit is associated to the item
 
 - 27 - unfiltered dispatching not supported for an Item, because a frequency prefiltering is applied for the item
 
 - 28 - unfiltered dispatching is not allowed by the current license terms (for special licenses only)
 
 - 66 - an unexpected exception was thrown by the Metadata Adapter while authorizing the connection
 
 - 68 - the Server could not fulfill the request because of an internal error.
 
 - <= 0 - the Metadata Adapter has refused the subscription or unsubscription request; the code value is dependent on the specific Metadata Adapter implementation
 
 @param code The error code sent by the Server.
 
 @param subscription The <LSSubscription> involved.
 
 @param message The description of the error sent by the Server; it can be nil.
 
 @param key The value of the key that identifies the second-level item.
 
 @see [LSConnectionDetails adapterSet]

 @see [LSSubscription commandSecondLevelFields]
 
 @see [LSSubscription commandSecondLevelFieldSchema]
 */
- (void) subscription:(nonnull LSSubscription *)subscription didFailWithErrorCode:(NSInteger)code message:(nullable NSString *)message forCommandSecondLevelItemWithKey:(nonnull NSString *)key;

/**
 @brief Event handler that is called by Lightstreamer to notify that all snapshot events for an item in the <LSSubscription> have been received, so that
 real time events are now going to be received.
 
 The received snapshot could be empty. Such notifications are sent only if the items are delivered in DISTINCT or COMMAND subscription mode and
 snapshot information was indeed requested for the items. By implementing this method it is possible to perform actions which require that all the initial 
 values have been received.
 
 Note that, if the involved <LSSubscription> has a two-level behavior enabled (see <[LSSubscription commandSecondLevelFields]> and <[LSSubscription commandSecondLevelFieldSchema]>),
 the notification refers to the first-level item (which is in COMMAND mode). Snapshot-related updates for the second-level items (which are in MERGE mode) 
 can be received both before and after this notification.
 
 @param subscription The <LSSubscription> involved.
 
 @param itemName The name of the involved item. If the Subscription was initialized using an "Item Group" then a nil value is supplied.
 
 @param itemPos The 1-based position of the item within the "Item List" or "Item Group".
 
 @see [LSSubscription requestedSnapshot]
 
 @see [LSItemUpdate snapshot]
 */
- (void) subscription:(nonnull LSSubscription *)subscription didEndSnapshotForItemName:(nullable NSString *)itemName itemPos:(NSUInteger)itemPos;

/**
 @brief Event handler that is called by Lightstreamer to notify that, due to internal resource limitations, Lightstreamer Server dropped one or more updates 
 for an item in the Subscription.
 
 Such notifications are sent only if the items are delivered in an unfiltered mode; this occurs if the subscription mode is:
 
 - `RAW`
 
 - `MERGE` or `DISTINCT`, with unfiltered dispatching specified
 
 - `COMMAND`, with unfiltered dispatching specified
 
 - `COMMAND`, without unfiltered dispatching specified (in this case, notifications apply to ADD and DELETE events only)
 
 By implementing this method it is possible to perform recovery actions.
 
 @param subscription The <LSSubscription> involved.
 
 @param lostUpdates The number of consecutive updates dropped for the item.
 
 @param itemName The name of the involved item. If the Subscription was initialized using an "Item Group" then a nil value is supplied.
 
 @param itemPos The 1-based position of the item within the "Item List" or "Item Group".
 
 @see [LSSubscription requestedMaxFrequency]
 */
- (void) subscription:(nonnull LSSubscription *)subscription didLoseUpdates:(NSUInteger)lostUpdates forItemName:(nullable NSString *)itemName itemPos:(NSUInteger)itemPos;

/**
 @brief Event handler that is called by Lightstreamer each time an update pertaining to an item in the <LSSubscription> has been received from the Server.
 
 @param subscription The <LSSubscription> involved.
 
 @param itemUpdate A value object containing the updated values for all the fields, together with meta-information about the update itself and some helper
 methods that can be used to iterate through all or new values.
 */
- (void) subscription:(nonnull LSSubscription *)subscription didUpdateItem:(nonnull LSItemUpdate *)itemUpdate;

/**
 @brief Event handler that receives a notification when the LSSubscriptionDelegate instance is removed from a <LSSubscription> through <[LSSubscription removeDelegate:]>.
 
 This is the last event to be fired on the delegate.
 
 @param subscription The <LSSubscription> this instance was removed from.
 */
- (void) subscriptionDidRemoveDelegate:(nonnull LSSubscription *)subscription;

/**
 @brief Event handler that receives a notification when the LSSubscriptionDelegate instance is added to a <LSSubscription> through
 <[LSSubscription addDelegate:]>.
 
 This is the first event to be fired on the delegate.
 
 @param subscription The <LSSubscription> this instance was added to.
 */
- (void) subscriptionDidAddDelegate:(nonnull LSSubscription *)subscription;

/**
 @brief Event handler that is called by Lightstreamer to notify that a <LSSubscription> has been successfully subscribed to through the Server.
 
 This can happen multiple times in the life of a <LSSubscription> instance, in case the Subscription is performed multiple times through
 <[LSLightstreamerClient unsubscribe:]> and <[LSLightstreamerClient subscribe:]>. This can also happen multiple times in case of automatic recovery after a connection
 restart.
 
 This notification is always issued before the other ones related to the same subscription. It invalidates all data that has been received previously.
 
 Note that two consecutive calls to this method are not possible, as before a second `subscriptionDidSubscribe:` event is fired an <subscriptionDidUnsubscribe:>
 event is eventually fired.
 
 If the involved <LSSubscription> has a two-level behavior enabled (see <[LSSubscription commandSecondLevelFields]> and <[LSSubscription commandSecondLevelFieldSchema]>),
 second-level subscriptions are not notified.
 
 @param subscription The <LSSubscription> involved.
 */
- (void) subscriptionDidSubscribe:(nonnull LSSubscription *)subscription;

/**
 @brief Event handler that is called when the Server notifies an error on a <LSSubscription>.
 
 By implementing this method it is possible to perform recovery actions.
 
 Note that, in order to perform a new subscription attempt, <[LSLightstreamerClient unsubscribe:]> and <[LSLightstreamerClient subscribe:]> should be issued
 again, even if no change to the <LSSubscription> attributes has been applied.
 
 The error code can be one of the following:
 
 - 15 - "key" field not specified in the schema for a COMMAND mode subscription
 
 - 16 - "command" field not specified in the schema for a COMMAND mode subscription
 
 - 17 - bad Data Adapter name or default Data Adapter not defined for the current Adapter Set
 
 - 21 - bad Group name
 
 - 22 - bad Group name for this Schema
 
 - 23 - bad Schema name
 
 - 24 - mode not allowed for an Item
 
 - 25 - bad Selector name
 
 - 26 - unfiltered dispatching not allowed for an Item, because a frequency limit is associated to the item
 
 - 27 - unfiltered dispatching not supported for an Item, because a frequency prefiltering is applied for the item
 
 - 28 - unfiltered dispatching is not allowed by the current license terms (for special licenses only)
 
 - 29 - RAW mode is not allowed by the current license terms (for special licenses only)
 
 - 30 - subscriptions are not allowed by the current license terms (for special licenses only)
 
 - 66 - an unexpected exception was thrown by the Metadata Adapter while authorizing the connection
 
 - 68 - the Server could not fulfill the request because of an internal error.
 
 - <= 0 - the Metadata Adapter has refused the subscription or unsubscription request; the code value is dependent on the specific Metadata
   Adapter implementation
 
 @param subscription The <LSSubscription> involved.
 
 @param code The error code sent by the Server.
 
 @param message The description of the error sent by the Server; it can be nil.
 
 @see [LSConnectionDetails adapterSet]
 */
- (void) subscription:(nonnull LSSubscription *)subscription didFailWithErrorCode:(NSInteger)code message:(nullable NSString *)message;

/**
 @brief Event handler that is called by Lightstreamer to notify that a <LSSubscription> has been successfully unsubscribed from.
 
 This can happen multiple times in the life of a <LSSubscription> instance, in case the <LSSubscription> is performed multiple times through
 <[LSLightstreamerClient unsubscribe:]> and <[LSLightstreamerClient subscribe:]>. This can also happen multiple times in case of automatic recovery after a
 connection restart.
 
 After this notification no more events can be received until a new <subscriptionDidSubscribe:> event.
 
 Note that two consecutive calls to this method are not possible, as before a second `subscriptionDidUnsubscribe:` event is fired an <subscriptionDidSubscribe:> event
 is eventually fired.
 
 If the involved <LSSubscription> has a two-level behavior enabled (see <[LSSubscription commandSecondLevelFields]> and <[LSSubscription commandSecondLevelFieldSchema]>),
 second-level unsubscriptions are not notified.

 @param subscription The <LSSubscription> involved.
 */
- (void) subscriptionDidUnsubscribe:(nonnull LSSubscription *)subscription;

/**
 @brief Event handler that is called by Lightstreamer to notify the client with the real maximum update frequency of the Subscription.
 
 It is called immediately after the Subscription is established and in response to a requested change (see <[LSSubscription requestedMaxFrequency]>).
 Since the frequency limit is applied on an item basis and a Subscription can involve multiple items,  this is actually the maximum frequency among all items. 
 For Subscriptions with two-level behavior enabled (see <[LSSubscription commandSecondLevelFields]> and <[LSSubscription commandSecondLevelFieldSchema]>),
 the reported frequency limit applies to both first-level and second-level items.
 
 The value may differ from the requested one because of restrictions operated on the server side,  but also because of number rounding.
 
 Note that a maximum update frequency (that is, a non-unlimited one) may be applied by the Server even when the subscription mode is RAW or
 the Subscription was done with unfiltered dispatching.
 
 @param subscription The <LSSubscription> involved.
 
 @param frequency A decimal number, representing the maximum frequency applied by the Server (expressed in updates per second), or the string `unlimited`.
 A nil value is possible in rare cases, when the frequency can no longer be determined.
 */
- (void) subscription:(nonnull LSSubscription *)subscription didReceiveRealFrequency:(nullable NSString *)frequency;


@end
